"""script for simple analysis of combined imaging using the data 
   obtained with Nobeyama Millimeter Array(NMA) and NRO 45-m telescope
   2011.09.29 by Y.Kurono (EA-ARC ALMA-J)
"""

### import interferometric visibility data
intfitsname = 'nma_visdata.demo.fits'
int_ms = intfitsname[:-5] +'.ms'
importuvfits(fitsfile = intfitsname, vis = int_ms)

#plotms(vis = int_ms, xaxis = 'uvdist', yaxis = 'amplitude')

### import single-dish image data
sdfitsname = '45m_imgdata.demo.fits'
sdimg = sdfitsname[:-5] +'.im'
importfits(fitsimage = sdfitsname, imagename = sdimg)

#viewer(infile = sdimg, gui = True)

### < only for Nobeyama data ---
""" change antenna diameter
    in the case of nobeyama millimeter array, uvfits processed are not 
    imported into CASA unless antenna table is added (already done).
    after import the visibility data, need to modify antenna information
    of dish diameter, because "If the telescope name is unknown, then the
    DISH DIAMETER in the ANTENNA table is used with a scaled VLA pattern.
"""
tb.open(int_ms+"/OBSERVATION", nomodify = False)
telscope_name = tb.getcol("TELESCOPE_NAME")
print ' telescope = ', telscope_name
tb.close()

tb.open(int_ms+"/ANTENNA", nomodify = False) 
diameter = tb.getcol("DISH_DIAMETER")
for i in range(len(diameter) -1):
    print ' change antenna diameter from ', diameter[i], ' to 10.'
    diameter[i] = 10.

tb.putcol("DISH_DIAMETER", diameter)
tb.close()
### --->

""" imaging with interferometric data only (CLEAN)
    better to use interactive mode
"""
clcutoff = '22mJy'
clean(vis = int_ms, imagename = int_ms +'.im', 
        mode = 'channel', 
        niter = 10000,
        gain = 0.1,
        threshold = clcutoff,
        #interactive = True, # comment out to use interactive mode
        imsize = [256, 256],
        cell = ['0.625arcsec'],
        weighting = 'natural',
        minpb = 0.1,
        calready = False,
        )
intclimg = int_ms +'.im' +'.image'
fluximg  = int_ms +'.im' +'.flux'

#viewer(infile = intclimg, gui = True)

""" sigle-dish image data
    adding stokes axis
""" 
reshaped_sdimg = sdimg +'.reshp'
ia.open(sdimg)
sd1 = ia.adddegaxes(stokes='I', overwrite = True)
sd1.reorder(outfile = reshaped_sdimg, order='0132')
sd1.done()
ia.close()

""" regridding
"""
regridded_sdimg = reshaped_sdimg +'.reg'
imregrid(imagename = reshaped_sdimg, 
        template = intclimg, 
        output = regridded_sdimg
        )

""" adding beam parameters to single-dish image
"""
sdbeamsize = '18.5arcsec'
imhead(regridded_sdimg, 
        mode = 'put', 
        hdkey = 'beammajor', 
        hdvalue = sdbeamsize
        )
imhead(regridded_sdimg, 
        mode = 'put', 
        hdkey = 'beamminor', 
        hdvalue = sdbeamsize
        )
imhead(regridded_sdimg, 
        mode = 'put', 
        hdkey = 'beampa', 
        hdvalue = '0deg'
        )

""" image unit conversion (K to Jy/bm)
    S_beam = 2*kb*Omega_beam/lambda^2 *Tb for single-dish image 
"""
lsp = 2.998e+10
kb = 1.381e-16
jy = 1.e-23
lmd = lsp /86.75433e+9
effmb = 0.5

sdbmaj_dic = imhead(regridded_sdimg, mode = 'get', hdkey = 'beammajor')
sdbmaj = sdbmaj_dic['value'] /3600. /180. *pi
sdbmin_dic = imhead(regridded_sdimg, mode = 'get', hdkey = 'beamminor')
sdbmin = sdbmin_dic['value'] /3600. /180. *pi
omega_sdbm = pi *sdbmaj *sdbmin /(4.*log(2))

sdfactor = 2. *kb *omega_sdbm /(lmd**2) /effmb /jy
print ' unit conversion factor: ', sdfactor

unitcnv_sdimg = regridded_sdimg +'.uc'
immath(imagename = regridded_sdimg, 
        expr = 'IM0*'+str(sdfactor), 
        outfile = unitcnv_sdimg,
        )

imhead(unitcnv_sdimg, 
        mode = 'put', 
        hdkey = 'bunit', 
        hdvalue = 'Jy/beam',
        )

""" apply primary beam of interferometric observation
"""
pbapp_sdimg = unitcnv_sdimg +'.pb'
immath(imagename = [unitcnv_sdimg, fluximg], 
        expr = 'IM0*IM1', 
        outfile = pbapp_sdimg,
        )

cleanwinimdl_cmbimg = int_ms +'.cbclean'
clcutoff = '67.0mJy'
clcutoff = '80.0mJy'
clean(vis = int_ms, imagename = cleanwinimdl_cmbimg, 
        mode = 'channel',
        niter = 10000,
        gain = 0.1,
        threshold = clcutoff,
        #interactive = True, # comment out to use interactive mode
        imsize = [256, 256],
        cell = ['0.625arcsec'],
        weighting = 'natural',
        modelimage = pbapp_sdimg, # using SD image as initial condition
        minpb = 0.1,
        calready = False,
        )
#viewer(infile = cleanwinimdl_cmbimg +'.image', gui = True)
#imstat(imagename = cleanwinimdl_cmbimg +'.image')
# END

""" feathering
"""
feather_cmbimg = int_ms +'.feather'
feather(feather_cmbimg, 
        highres = intclimg, 
        lowres = pbapp_sdimg,
        )


#create difference maps;
immath(imagename = [feather_cmbimg, cleanwinimdl_cmbimg +'.image'], 
        expr = 'IM0-IM1', 
        outfile = 'feath.sub.cln',
        )
viewer('feath.sub.cln')
