os.system('clear')
print '################################################################################################'
print 'Script for simple analysis of combined imaging using the data'
print 'obtained with Nobeyama Millimeter Array(NMA) and NRO 45-m telescope'
print '2011.09.29 by Y.Kurono (EA-ARC ALMA-J)'
print '################################################################################################'
print ''
### Stage 1: import interferometric visibility data
print 'Stage 1: import interferometric visibility data. Press return to continue...'
a=raw_input()
print 'Importing...'
intfitsname = 'nma_visdata.demo.fits'
int_ms = intfitsname[:-5] +'.ms'
importuvfits(fitsfile = intfitsname, vis = int_ms)

plotms(vis = int_ms, xaxis = 'uvdist', yaxis = 'amplitude')
print 'Import complete - please observe visibility amplitudes in displayed plot & press return to continue.'
a=raw_input()
os.system('clear')

### Stage 2:  import single-dish image data
print '################################################################################################'
print 'Stage 2:  import single-dish image data. Press return to continue...'
a=raw_input()
print 'Importing...'
sdfitsname = '45m_imgdata.demo.fits'
sdimg = sdfitsname[:-5] +'.im'
importfits(fitsimage = sdfitsname, imagename = sdimg)

viewer(infile = sdimg, gui = True)
print 'Import complete - please observe image displayed in viewer & press return to continue.'
a=raw_input()
os.system('clear')

### Stage 3. Apply fixes & corrections to Nobeyama data headers
print '################################################################################################'
print 'Stage 3. Apply fixes & corrections to Nobeyama data headers:'
print 'Changing antenna diameter:'
print 'in the case of nobeyama millimeter array, uvfits processed are not' 
print 'imported into CASA unless antenna table is added (already done).'
print 'after import the visibility data, need to modify antenna information'
print 'of dish diameter, because "If the telescope name is unknown, then the'
print 'DISH DIAMETER in the ANTENNA table is used with a scaled VLA pattern.'
print ''
print 'Press return to continue...'
a=raw_input()

tb.open(int_ms+"/OBSERVATION", nomodify = False)
telscope_name = tb.getcol("TELESCOPE_NAME")
print ' telescope = ', telscope_name
tb.close()

tb.open(int_ms+"/ANTENNA", nomodify = False) 
diameter = tb.getcol("DISH_DIAMETER")
for i in range(len(diameter) -1):
    print ' change antenna diameter from ', diameter[i], ' to 10.'
    diameter[i] = 10.

tb.putcol("DISH_DIAMETER", diameter)
tb.close()
### Stage 4. imaging with interferometric data only (CLEAN). It's better to use interactive mode
print ''
print ''
print '################################################################################################'
print 'Stage 4. imaging with interferometric data only (CLEAN). Press return to continue...'
print 'Note  - this stage will generate warnings.. they can be ignored'
a=raw_input()
clcutoff = '22mJy'
clean(vis = int_ms, imagename = int_ms +'.im', 
        mode = 'channel', 
        niter = 10000,
        gain = 0.1,
        threshold = clcutoff,
        #interactive = True, # comment out to use interactive mode
        imsize = [256, 256],
        cell = ['0.625arcsec'],
        weighting = 'natural',
        minpb = 0.1,
        calready = False,
        )
intclimg = int_ms +'.im' +'.image'
fluximg  = int_ms +'.im' +'.flux'

viewer(infile = intclimg, gui = True)
print ''
print ''
print 'Clean complete - please observe image displayed in viewer & press return to continue.'
a=raw_input()
os.system('clear')

###Stage 5. Add stokes axis to single-dish data
print '################################################################################################'
print 'Stage 5. Add stokes axis to single-dish data, regridding & modification to SD header parameters. Press return to continue...'
reshaped_sdimg = sdimg +'.reshp'
a=raw_input()
ia.open(sdimg)
sd1 = ia.adddegaxes(stokes='I', overwrite = True)
sd1.reorder(outfile = reshaped_sdimg, order='0132')
sd1.done()
ia.close()


print 'Stage 5 - done'

###Stage 6. regridding single-dish data to enable Feathering & for use as model in CLEAN
print ''
print ''
print '################################################################################################'
print 'Stage 6. regridding single-dish data to enable Feathering & for use as model in CLEAN'
print 'Note, this stage will generate warnings - they can be ignored.'
regridded_sdimg = reshaped_sdimg +'.reg'
imregrid(imagename = reshaped_sdimg, 
        template = intclimg, 
        output = regridded_sdimg
        )
### add beam parameters to single-dish image
print 'adding beam parameters to single-dish image...'
sdbeamsize = '18.5arcsec'
imhead(regridded_sdimg, 
        mode = 'put', 
        hdkey = 'beammajor', 
        hdvalue = sdbeamsize
        )
imhead(regridded_sdimg, 
        mode = 'put', 
        hdkey = 'beamminor', 
        hdvalue = sdbeamsize
        )
imhead(regridded_sdimg, 
        mode = 'put', 
        hdkey = 'beampa', 
        hdvalue = '0deg'
        )

print 'image unit conversion (K to Jy/bm)'
print 'S_beam = 2*kb*Omega_beam/lambda^2 *Tb for single-dish image'

lsp = 2.998e+10
kb = 1.381e-16
jy = 1.e-23
lmd = lsp /86.75433e+9
effmb = 0.5

sdbmaj_dic = imhead(regridded_sdimg, mode = 'get', hdkey = 'beammajor')
sdbmaj = sdbmaj_dic['value'] /3600. /180. *pi
sdbmin_dic = imhead(regridded_sdimg, mode = 'get', hdkey = 'beamminor')
sdbmin = sdbmin_dic['value'] /3600. /180. *pi
omega_sdbm = pi *sdbmaj *sdbmin /(4.*log(2))

sdfactor = 2. *kb *omega_sdbm /(lmd**2) /effmb /jy
print ' unit conversion factor: ', sdfactor

unitcnv_sdimg = regridded_sdimg +'.uc'
immath(imagename = regridded_sdimg, 
        expr = 'IM0*'+str(sdfactor), 
        outfile = unitcnv_sdimg,
        )

imhead(unitcnv_sdimg, 
        mode = 'put', 
        hdkey = 'bunit', 
        hdvalue = 'Jy/beam',
        )
print ''
print 'Stage 6 - done  Press return to continue...'
a=raw_input()
os.system('clear')

###Stage 7: apply interferometer primary beam to Single Dish dataset.
print '################################################################################################'
print 'Stage 7: apply interferometer primary beam to Single Dish dataset.'
pbapp_sdimg = unitcnv_sdimg +'.pb'
immath(imagename = [unitcnv_sdimg, fluximg], 
        expr = 'IM0*IM1', 
        outfile = pbapp_sdimg,
        )
print 'Stage 7 - done.'

###Stage 8. Feathering
print ''
print ''
print '################################################################################################'
print 'Stage 8: feathering. Press return to continue...'
print 'Note, this stage will generate warnings - they can be ignored.'
a=raw_input()
feather_cmbimg = int_ms +'.feather'
feather(feather_cmbimg, 
        highres = intclimg, 
        lowres = pbapp_sdimg,
        )

viewer(infile = feather_cmbimg, gui = True)
print ''
print ''
print 'Feathering complete - please observe combined image in viewer. Press return to continue.'
a=raw_input()
os.system('clear')

###Stage 9: Alternative combination method: clean with sd image as modelimage.
print '################################################################################################'
print 'Stage 9: Alternative combination method: CLEAN with sd image as model image.  Press return to continue...'
print 'Note, this stage will generate warnings - they can be ignored.'
a=raw_input()
cleanwinimdl_cmbimg = int_ms +'2.cbclean'
clcutoff = '70mJy'
clean(vis = int_ms, imagename = cleanwinimdl_cmbimg, 
        mode = 'channel', 
        niter = 10000,
        gain = 0.1,
        threshold = clcutoff,
        #interactive = True, # comment out to use interactive mode
        imsize = [256, 256],
        cell = ['0.625arcsec'],
        weighting = 'natural',
        modelimage = pbapp_sdimg, # using SD image as initial condition
        minpb = 0.1,
        calready = False,
        )

viewer(infile = cleanwinimdl_cmbimg +'.image', gui = True)
print 'CLEAN + SD model complete - please observe combined image in viewer.'
print 'Script ends. Have a nice day'
# END



